from pathlib import Path
from typing import Union

def check_obj_list(obj_list: Union[dict, list[dict]]):
    if isinstance(obj_list, dict):
        obj_list = [obj_list]
    elif not isinstance(obj_list, list):
        raise TypeError(f"obj_list must be a dict or list[dict]; got {type(obj_list).__name__}")
    
    for i, obj in enumerate(obj_list):
        if not isinstance(obj, dict):
            raise TypeError(f"obj_list must be list[dict]; got {type(obj).__name__} at index {i}")

def check_file(file: Union[str, Path]):
    if not isinstance(file, str):
        raise TypeError("file must be str, got {}".format(type(file).__name__))



from pathlib import Path
from typing import Union
import os
import json
from .type_checker import check_file, check_obj_list

def read_jsonl(file: Union[str, Path]):
    check_file(file)
    
    df = []
    with open(file, mode='r', encoding='utf-8') as reader:
        line = reader.readline()
        while line:
            obj = json.loads(line)
            df.append(obj)
            line = reader.readline()
    return df

def write_jsonl(obj_list: Union[dict, list[dict]], file: Union[str, Path], force=False, silent=False):
    check_obj_list(obj_list)
    check_file(file)
    
    if os.path.exists(file) and force == False:
        print('[INFO] {} already exists.'.format(file))
        return
    
    dir_path = os.path.dirname(file)
    if dir_path != '':
        os.makedirs(dir_path, exist_ok=True)
    
    with open(file, mode='w', encoding='utf-8') as fp:
        for obj in obj_list:
            json.dump(obj, fp, ensure_ascii=False)
            print(file=fp)
    
    if not silent:
        print('[INFO] save to {}'.format(file))


def append_jsonl(obj_list: Union[dict, list[dict]], file: Union[str, Path]) -> None:
    check_obj_list(obj_list)
    check_file(file)
    
    dir_path = os.path.dirname(file)
    if dir_path != '':
        os.makedirs(dir_path, exist_ok=True)
    
    with open(file, mode='a', encoding='utf-8') as fp:
        for obj in obj_list:
            json.dump(obj, fp, ensure_ascii=False)
            print(file=fp)


from .jsonl_utils import (
    read_jsonl,
    write_jsonl,
    append_jsonl
)

from .json_utils import (
    read_json,
    write_json,
)

from .pickle_utils import (
    load_pickle,
    save_pickle,
)


from pathlib import Path
from typing import Union
import os
from .type_checker import check_file

import pickle
def load_pickle(file: Union[str, Path]):
    check_file(file)

    with open(file, 'rb') as f:
        obj = pickle.load(f)
    return obj

def save_pickle(obj, file: Union[str, Path], force=False, silent=False):
    check_file(file)

    if os.path.exists(file) and force == False:
        print('[INFO] {} already exists.'.format(file))
        return
    
    dir_path = os.path.dirname(file)
    if dir_path != '':
        os.makedirs(dir_path, exist_ok=True)
    
    with open(file, 'wb') as f:
        pickle.dump(obj, f)
    
    if not silent:
        print('[INFO] save to {}'.format(file))



from pathlib import Path
from typing import Union
import os
import json
from .type_checker import check_file

def read_json(file: Union[str, Path]):
    check_file(file)

    with open(file, mode='r', encoding='utf-8') as reader:
        df = json.load(reader)
    return df

def write_json(obj: Union[dict], file: Union[str, Path], force=False, silent=False):
    check_file(file)
    if not isinstance(obj, dict):
        raise TypeError("obj must be dict, got {}".format(type(obj).__name__))

    if os.path.exists(file) and force == False:
        print('[INFO] {} already exists.'.format(file))
        return
    
    dir_path = os.path.dirname(file)
    if dir_path != '':
        os.makedirs(dir_path, exist_ok=True)

    with open(file, mode='w', encoding='utf-8') as fp:
        json.dump(obj, fp, ensure_ascii=False, indent=4)
        print(file=fp)
    
    if not silent:
        print('[INFO] save to {}'.format(file))
